from typing import Type

from snowman.query._builder._delete_query_builder import DeleteQueryBuilder
from snowman.query._builder._insert_query_builder import InsertQueryBuilder
from snowman.query._builder._truncate_query_builder import TruncateQueryBuilder
from snowman.query._builder._update_query_builder import UpdateStatement
from snowman.relation.table import (
    GenericInsertColumnTypedDict,
    GenericUpdateColumnTypedDict,
    Table,
)

insert = InsertQueryBuilder()


def update(
    table: Type[Table[GenericInsertColumnTypedDict, GenericUpdateColumnTypedDict]],
):
    return UpdateStatement(
        table,
        _columns_type=table.__update_columns__,  # type: ignore
    )


delete = DeleteQueryBuilder()

truncate = TruncateQueryBuilder()


__all__ = [
    "insert",
    "update",
    "delete",
    "truncate",
]



import importlib
import importlib.metadata

from . import datatype as datatype
from . import query as query
from .relation.table import Table as Table
from .relation.table import table as table

__version__ = importlib.metadata.version("snowman-py")



from typing import Generic, Type
from typing_extensions import override

from snowman.relation import full_table_name
from snowman.relation.table import GenericTable

from ._builder import QueryBuilder, QueryWithParams


class DeleteQueryBuilder:
    def from_(self, table: Type[GenericTable]) -> "DeleteFromStatement[GenericTable]":
        return DeleteFromStatement(table)


class DeleteFromStatement(Generic[GenericTable]):
    def __init__(self, table: Type[GenericTable]):
        self._table = table

    def where(self, condition: str) -> "UpdateFromWhereQueryBuilder[GenericTable]":
        return UpdateFromWhereQueryBuilder(self._table, condition)


class UpdateFromWhereQueryBuilder(Generic[GenericTable], QueryBuilder):
    def __init__(self, table: Type[GenericTable], where_condition: str):
        self._table = table
        self._where_condition = where_condition

    @override
    def build(self) -> QueryWithParams:
        query = f"""
DELETE FROM
    {full_table_name(self._table)}
WHERE
    {self._where_condition}
""".strip()

        return QueryWithParams(query, {})
